import re
import unicodedata

# Dictionary copied from DNAconvert:
# repository: https://github.com/iTaxoTools/DNAconvert/
# source file: src/itaxotools/DNAconvert/library/ext_ASCII_conv_table.py

ext_ascii_dict = {
    ord("ƒ"): "f",
    ord("Š"): "S",
    ord("Œ"): "OE",
    ord("Ž"): "Z",
    ord("š"): "s",
    ord("œ"): "oe",
    ord("ž"): "z",
    ord("Ÿ"): "Y",
    ord("¡"): "i",
    ord("¢"): "c",
    ord("ª"): "a",
    ord("²"): "2",
    ord("³"): "3",
    ord("µ"): "u",
    ord("¹"): "1",
    ord("º"): "o",
    ord("À"): "A",
    ord("Á"): "A",
    ord("Â"): "A",
    ord("Ã"): "A",
    ord("Ä"): "Ae",
    ord("Å"): "A",
    ord("Æ"): "Ae",
    ord("Ç"): "C",
    ord("È"): "E",
    ord("É"): "E",
    ord("Ê"): "E",
    ord("Ë"): "E",
    ord("Ì"): "I",
    ord("Í"): "I",
    ord("Î"): "I",
    ord("Ï"): "I",
    ord("Ð"): "D",
    ord("Ñ"): "N",
    ord("Ò"): "O",
    ord("Ó"): "O",
    ord("Ô"): "O",
    ord("Õ"): "O",
    ord("Ö"): "Oe",
    ord("×"): "x",
    ord("Ø"): "O",
    ord("Ù"): "U",
    ord("Ú"): "U",
    ord("Û"): "U",
    ord("Ü"): "Ue",
    ord("Ý"): "Y",
    ord("ß"): "ss",
    ord("à"): "a",
    ord("á"): "a",
    ord("â"): "a",
    ord("ã"): "a",
    ord("ä"): "ae",
    ord("å"): "a",
    ord("æ"): "a",
    ord("ç"): "c",
    ord("è"): "e",
    ord("é"): "e",
    ord("ê"): "e",
    ord("ë"): "e",
    ord("ì"): "i",
    ord("í"): "i",
    ord("î"): "i",
    ord("ï"): "i",
    ord("ð"): "d",
    ord("ñ"): "n",
    ord("ò"): "o",
    ord("ó"): "o",
    ord("ô"): "o",
    ord("õ"): "o",
    ord("ö"): "oe",
    ord("ù"): "ue",
    ord("ú"): "ue",
    ord("û"): "ue",
    ord("ü"): "ue",
    ord("ý"): "y",
    ord("ÿ"): "y",
}
ext_ascii_trans = str.maketrans(ext_ascii_dict)


def sanitize(text: str) -> str:
    """Replaces special characters in a string with underscores"""
    text = unicodedata.normalize("NFKC", text).translate(ext_ascii_trans)
    text = re.sub(r"^[^\w ]+", "", text)
    return re.sub(r"[^\w ]+", "_", text)
